local addonName, TF = ...

local TradeFill = LibStub("AceAddon-3.0"):GetAddon(addonName)
local Loc = LibStub("AceLocale-3.0"):GetLocale(addonName)

function TradeFill:TRADE_SHOW()
    if not TradeFrame:IsVisible() then
        CancelTrade()

        return
    else
        local target, _ = UnitName("npc")
        local className, classFile, _ = UnitClass("npc")

        TF.name = target
        TF.level = UnitLevel("npc")
        TF.guild = GetGuildInfo("npc")
        TF.class = { ["name"] = className, ["file"] = classFile }
        TF.item = TradeFill.db.profile.item
        TF.stack = TradeFill.db.profile.stacks[classFile]
        TF.setting = TradeFill.db.profile.setting

        self:RegisterEvent("TRADE_CLOSED")
        self:RegisterEvent("UI_INFO_MESSAGE")
        self:RegisterEvent("BAG_UPDATE_DELAYED")
        self:RegisterEvent("SECURE_TRANSFER_CANCEL")
        self:RegisterEvent("TRADE_ACCEPT_UPDATE")

        TradeFill:SearchInventory()
        TradeFill:ButtonInitialize()

        local group = TradeFill:GroupCondition()

        local stranger = TradeFill.db.profile.setting.solo
        local party = TradeFill.db.profile.setting.party
        local raid = TradeFill.db.profile.setting.raid

        if ( stranger == true and group.status == "solo" ) then

            if TradeFill:TradingCondition() then
                return
            else
                for key = 1, MAX_TRADABLE_ITEMS do
                    local index = tostring(key)

                    if TF.item[index].id > 0 and TF.stack[index] > 0 then
                        TradeFill:ItemCondition(TF.stack[index], TF.item[index])
                    end
                end
                return
            end
        end

        if ( party == true and group.status == "party") or (raid == true and group.status == "raid" ) then

            if TradeFill:TradingCondition() then
                return
            else
                local numGroupMembers = GetNumGroupMembers()

                if numGroupMembers > 0 then
                    for i = 1, numGroupMembers do
                        local unitName, _ = UnitName(group.status .. i)

                        if unitName == TF.name then

                            for key = 1, MAX_TRADABLE_ITEMS do
                                local index = tostring(key)

                                if TF.item[index].id > 0 and TF.stack[index] > 0 then
                                    TradeFill:ItemCondition(TF.stack[index], TF.item[index])
                                end
                            end
                            break
                        end
                    end
                end
            end
        end
    end
end

function TradeFill:SetName()
    local class = TF.class.file
    local _, _, _, argbHex = GetClassColor(class)

    if class == "SHAMAN" then
        argbHex = "FF0070DE"
    end

    return "|c".. argbHex .. TF.name .. "|r"
end

function TradeFill:GroupCondition()
    if IsInRaid() then
        return { ["status"] = "raid" }
    elseif IsInGroup() then
        return { ["status"] = "party" }
    else
        return { ["status"] = "solo" }
    end
end

function TradeFill:TRADE_ACCEPT_UPDATE(event, sender, receiver)
    if sender == 1 or receiver == 1 then
        TradeFill:getTradeHistory()
    end
end

function TradeFill:UI_INFO_MESSAGE(event, index, info)
    if info == 	"Trade complete."  then
        TradeFill:Message(string.format(Loc["TRADE_COMPLETE"], TradeFill:SetName()), false)
        TradeFill:setTradeHistory()
    end

    if info == "Trade cancelled." then
        TradeFill:Message(string.format(Loc["TRADE_CANCELED"], TradeFill:SetName()), false)
    end
end

function TradeFill:BAG_UPDATE_DELAYED()
    if TradeFrame:IsVisible() then
        TradeFill:ClearTade()

        TradeFill:TRADE_SHOW()
    else
        if next(TF.interfaceItem) ~= nil then
            TF.interfaceItem = {}
        end
    end
end

function TradeFill:SECURE_TRANSFER_CANCEL()
    for clearSlot = 1, MAX_TRADABLE_ITEMS do
        TF.stackCount[clearSlot].trade = 0
    end

    for slot = 1, MAX_TRADABLE_ITEMS do
        if GetTradePlayerItemLink(slot) ~= nil then
            local name, _, quantity, _, _, _ = GetTradePlayerItemInfo(slot)
            local id, _, _, _, _, _, _ = GetItemInfoInstant(GetTradePlayerItemLink(slot))

            if name ~= nil then
                for key = 1, MAX_TRADABLE_ITEMS do
                    if TF.stackCount[key].id == id then
                        TF.stackCount[key].trade = TF.stackCount[key].trade + quantity
                    end
                end
            end
        end
    end

    TradeFill:ButtonUpdate()
end

function TradeFill:TRADE_CLOSED()
    TF.messageAppear = {}

    TradeFill:UnregisterEvent("SECURE_TRANSFER_CANCEL")
end